// See: https://andreaskroepelin.github.io/polylux/book/polylux.html
#import "@preview/polylux:0.4.0": *
#import "@preview/showybox:2.0.3": showybox
// see https://sitandr.github.io/typst-examples-book/book/packages/boxes.html
#import "@preview/codelst:2.0.2": sourcecode

#set page(paper: "presentation-16-9")
#set text(size: 25pt)
#set text(lang: "de")
#let framecolors = (
  border-color: blue,
  title-color: blue.lighten(30%),
  body-color: blue.lighten(95%),
  footer-color: blue.lighten(80%)
)
#let now() = { // Feb 3, 2025
  set text (lang: "de")
  let today = datetime.today()
  return [#today.display("[month repr:long] [day padding:none], [year] ")]
}
#let overlay(overlay_text, y_offset: -3cm, bg_color:red,
  ) = {
    place(right,
          dx: 0cm, dy:y_offset,
          rotate(20deg,
            block(
                fill: bg_color, outset: 0.4cm, radius: 0.3cm,
                overlay_text
            )
          )
    )
}



// Footnotes: smaller with no indent:
#set footnote.entry(indent: 0em)
#show footnote.entry: set text(size: 0.8em)


//
// Title
//
#set page(footer: align(right, text(size: 12pt, stroke:gray, now())))
#slide[
  #set align(horizon)
  #heading(outlined: false, [
      Crash Course
  ])
  #align(right,image("figures/py.svg",width: 8%))
  von H. Schwab und J. Feiner #footnote[FH JOANNEUM, IIT] #h(1em)

]

// Footer should be rather small
#set page(footer: text(size: 12pt,[Python Crash Course]))

// More space after each slide title
#show heading: set block(below: 2em)

// - - - - - - -

//
// Agenda
//
#slide[

  #outline( title: "Agenda",depth: 2)

]

#slide[
	== Spaß am Programmieren?

	- Fragen zur Motivation:
  	- warum Programmieren?
  	- warum Automatisieren?
  	- warum Python?
  	- ... und wie kann Coden möglichst viel Spaß machen?
]
#slide[
	=== Spaß am Programmieren? Meine Antworten

  - Mehrwert!
    - Skript um wiederholte Arbeiten nur einmal zu tun
      - = Faulsein?
      - = Wiederverwendbarkeit! = Dokumentation!
    - Automatisierung die auch Kolleg:innen nutzen können
  - Freude beim "Rätsellösen"
    - Schnelle Erfolge mit Python
    - Code ist einfach u. prägnant, les- und dadurch wartbar, flexibel und mächtig (libs), ..Community!

]
#slide[
  === Spaß am Programmieren?

  Aber ich weiss auch, ...

    ... ich schreibe bloss kleine Scripts
  und keine komplette Applikation oder ein Framework/Library/API.
]
#slide[
	== Wie starte ich beim Code Erstellen?

	- Erste (einfache) Python Programme
	 - Ablauf (Algorithmus) als Kommentar
		#showybox(
          title: "Design your algoritm",
          frame: framecolors,
          footer: "",
        				raw(  read("code/comments.py"), lang:"python")
        )


]
#slide[
	=== Typische Beispiele (1/4) - Statischer Web Server
	#showybox(
          title: "Provide data from './data' directory via static web server",
          frame: framecolors,
          footer: "Try in Browser 'http://localhost:8080/tree.png'.",
        				raw(  read("code/mini-1.sh"), lang:"python")
        )

]
#slide[
	=== Typische Beispiele (2/4) Daten vom Server laden
		    #showybox(
          title: "Web: download into 'images' folder",
          frame: framecolors,
          footer: "",
        				raw(  read("code/mini-2.py"), lang:"python")
        )
       #overlay([Please improve#footnote("Diskussion: Lesbarkeit, Fehlerbehandlung, Flexibilität, Stabilität, Voraussetzungen, Nützlichkeit")], y_offset:-6cm)
]
#slide[
	=== Typische Beispiele (3/4) Sicherungen anlegen
     #showybox(
          title: "Backup files of dir 'images'",
          frame: framecolors,
          footer: "Check files with timestamp in subdir 'backup'.",
        				raw(  read("code/mini-3.py"), lang:"python")
        )
        #overlay([How to improve?], y_offset: -7cm)
]

#slide[
	=== Typische Beispiele (4/4) - Textdatei Analyse
       #showybox(
          title: "Analyse lines in 'data.log'",
          frame: framecolors,
          footer: "Check reformatted, filtered 'warn.csv' file.",
        				raw(  read("code/mini-4.py"), lang:"python")
        )
        #overlay(align(left,text(size: 14pt, [2025-01-29 WARN System down #linebreak()
                  2025-01-30 INFO System restart#linebreak()
                  2025-02-02 WARN System down again#linebreak()
                  2025-02-02 ERR Kernel Panic#linebreak()
                  ...
                  ])), y_offset: -9.5cm, bg_color: gray)
        #overlay(align(left,text(size: 14pt, [
                  System down,(2025-01-29)#linebreak()
                  System down again,(2025-02-02)#linebreak()
                  ])), y_offset: -0cm, bg_color: gray)
]




#slide[
	== Wie helfe ich mir selbst weiter?

	- Unterstützung
  	- REPL, Terminal
  	- Entwicklungsumgebung (IDEs: VS Code, PyCharm)
  	- Debugging
  - Hilfe Online (Python Docs, Stackoverflow, AI)
	- Syntax (Coding Conventions, PEP8)
]
#slide[
	=== REPL
  #showybox(
          title: "Read Evaluate Print Loop",
          frame: framecolors,
          footer: "",
        				raw(  read("code/repl.py"), lang:"python")
        )
  ]
#slide[
	=== Syntax
  #showybox(
          title: "Syntax: datatypes, functions, conditions, loops",
          frame: framecolors,
          footer: "",
        				raw(  read("code/syntax.py"), lang:"python")
        )
  ]
  Tipp: sehr hilfreich ist der zweiseitige Python Schwindelzettel#footnote(
    link("https://perso.limsi.fr/pointal/_media/python:cours:mementopython3-german.pdf")
  ).

    #slide[
  == Schnelle Erfolge

  - Erweitern der Einzeiler
    - Input/Output Handling
    - Interaktiver User-Input
    - Dateien verarbeiten
]
#slide[
  == Ausblick

  - Wartbarkeit (für mich und andere Entwickler:innen)
    - Syntax und Code Konventionen
    - Python Virtual Environment einrichten
  - IDE mit AI einrichten
]

#slide[
	=== Virtual Environments
  #showybox(
          title: "Virtual environment",
          frame: framecolors,
          footer: "",
        				raw(  read("code/venv.sh"), lang:"python")
        )
  ]
