#!/usr/bin/env python3

import sys
from os import makedirs
from urllib.request import urlretrieve

# Set defaults:
out_dir = "images"

print(f"Downloading images into folder '{out_dir}'...")

if len(sys.argv) < 2:
    sys.exit(f"\tUsage: {sys.argv[0]} <file> [<folder>].")
filename = sys.argv[1]

# Optionally, the user specified the target folder:
if len(sys.argv) > 2:
    out_dir = sys.argv[2]

# create target dir and do not complain if it exists
makedirs(out_dir, exist_ok=True)

# numbers to name the images: e.g. image-0013.jpg
next_number = 0
with open(filename) as fh:
    urls = set(fh.readlines())
    for idx, url in enumerate(urls, start=1):
        url = url.strip()
        suffix = url.split(".")[-1]
        local_name = f"{out_dir}/image-{next_number + 1:04}.{suffix}"
        print(f"\t{idx}/{len(urls)} downloading '{url}' to {local_name}", end="...")
        try:
            urlretrieve(url, local_name)
            next_number += 1
            print("OK")
        except Exception:
            print("NOT OK")

print(f"Done. Find {next_number} downloaded pictures in directory '{out_dir}'.")
