#!/usr/bin/env python3
import getpass
import http.client
import json
import os
import sys

# ./downloader.py https://myserver:8099/ws/v2/users.json

print("Downloading text resources from server:")

# To perform local tests, you might start a mini-server:
# run:
#     SERVER="localhost" ./speakyserver.py
# run
#     export SERVER="127.0.0.1"
#     ./speakyserver.py


# Low prio
# Setting some defaults
proto = "http"
port = 8090
resource = "secure.txt"

# Medium prio
# Checking if environment is set
server = os.environ.get("SERVER", "127.0.0.1")
username = os.environ.get("USER", "admin")
port = os.environ.get("PORT", port)
# print(f"DBG: PORT='{port}'")

# High prio
# Read config file (if available). It contains e.g.: PORT=9090
try:
    import config

    port = config.PORT
except Exception as err:
    print(f"ERR: {err}")
    print("\tDBG: no config file 'config.py' given.")
    print("\tDBG: we proceed without configuration from file 'config.py'.")


# Higher prio
# Do we get commandline parameters
if len(sys.argv) > 1:
    try:
        proto, _, serverwithport, resource = sys.argv[1].split("/", maxsplit=3)
        proto = proto.strip(":")
        # print(f"DBG: {sys.argv[1].split('/')}")
        if serverwithport.count(":"):
            server, port = serverwithport.split(":")
        else:
            server, port = serverwithport, port
        url = f"{proto}://{server}:{port}/{resource}"
    except Exception as err:
        print(f"ERR: {err}")
        print(f"\tUsage {sys.argv[0]} <url>")
        exit(1)

# We got everything (but the passwort)
print(f"Downloading '{resource}' via '{proto}' from '{server}' (using port '{port}').")


# Very high prio
# interactive input
# hide prompt for passwords(!)
password = getpass.getpass(f"Please specify the password for user {username}: ")


# startup
conn = http.client.HTTPConnection(server, int(port))
headers = {"Content-type": "application/json"}
data = {"user": username, "password": password}
try:
    conn.request("POST", f"/{resource}", json.dumps(data), headers)
except Exception as err:
    print(f"ERR. Sorry request could not be performed (ERR '{err}')")
    print(f"\tEnsure that {server}:{port} can be reached over the network.")
    print("\tThen try again.")
    exit(1)

print(f"DBG: {conn}")
res = conn.getresponse()
if res.status == 200:
    msg = res.read().decode("utf-8").strip()
    print(f" We got response from server: '{msg}'")
else:
    print("ERR. Check if the url is available on the server.")


# Security - Discussion:
#  Q "Is ANY solution using username/password optimal?"
#  A   api keys
#  A   tokens (one time, lifetime)
#  A   two factor (sms, smartphone biometrics)
