#!/usr/bin/env python3

import json
from http.server import BaseHTTPRequestHandler, HTTPServer
from urllib.parse import urlparse

# try
# a) start server
#    /speakyserver.py
# b) get info
#    curl 'http://localhost:8000/ws/apiv3/user/admin/info?lang=en&log=true#page=9'
#    curl   -X POST \
#           --data "lang=en"\
#           --data "log=true"\
#           -H "Content-Type: application/x-www-form-urlencoded"\
#           'http://localhost:8000/ws/apiv3/user/admin/info'


class MyChattyMiniServer(BaseHTTPRequestHandler):
    def _set_response_headers(self):
        self.send_response(200)
        self.send_header("Content-type", "application/json")
        self.end_headers()

    def log_info_to_console(self, raw_path):
        parsed_path = urlparse(raw_path)
        print(f"DBG: parsed path: '{parsed_path}'")
        print(f"* Log: path: '{parsed_path.path}'")
        print(f"* Log: params: '{parsed_path.params}'")
        print(f"* Log: query: '{parsed_path.query}'")
        print(f"* Log: fragment: '{parsed_path.fragment}'")

    def do_GET(self):
        self._set_response_headers()
        self.log_info_to_console(self.path)
        response = {"status": "ok", "method": "GET"}
        self.wfile.write(json.dumps(response).encode())

    def do_POST(self):
        self._set_response_headers()
        self.log_info_to_console(self.path)
        response = {"status": "ok", "method": "POST"}
        self.wfile.write(json.dumps(response).encode())

    def do_HEAD(self):
        self._set_response_headers()


def run(server_class=HTTPServer, handler_class=MyChattyMiniServer, port=8000):
    server_address = ("", port)
    httpd = server_class(server_address, handler_class)
    print(f"Starting local server (httpdaemon) at port {port}...")
    httpd.serve_forever()


if __name__ == "__main__":
    from sys import argv

    if len(argv) == 2:
        run(port=int(argv[1]))
    else:
        run()
