#!/bin/bash
echo "Setting up Python virtual environment to later install libraries/packages..."
python3 -m venv .venv
echo " * Activating the virtual environment (deactivate with 'deactivate')..."
source .venv/bin/activate
echo " * Installing all libraries listed in 'requirements.txt' file into virtual environment..."
pip install -r ./requirements.txt


echo " * Starting up your Python programme now..."
# Note: instead of 'python imgapi.py', we use:

# Following configuration will be available in Flask with:
# imgapi.config["IMGAPI"]["admin"]["password"]
export FLASK_IMGAPI__admin__passwordhash="ac15ae65cac19d8d1fb17ef44c9f15ce0e459384a90e99bae9d7b33fb0d6beba" 

# FYI: we can hash 'ThisIsAVery8ecureAndLongPa$$word!' in Python REPL
# >>> import hashlib
# >>> def sha256(plain: str):
# ...     m = hashlib.sha256()
# ...     m.update(plain.strip().encode('utf8'))
# ...     return m.hexdigest()
# ... 
# >>> sha256("ThisIsAVery8ecureAndLongPa$$word!")
# 'ac15ae65cac19d8d1fb17ef44c9f15ce0e459384a90e99bae9d7b33fb0d6beba'

# if sha256sum or sha3sum is available on your system:

# echo -n 'ThisIsAVery8ecureAndLongPa$$word!' | sha256sum | cut -d' ' -f1       <= Linux Debian
#  ac15ae65cac19d8d1fb17ef44c9f15ce0e459384a90e99bae9d7b33fb0d6beba             <= Linux Debian
# But take care with macOS:
# echo -n 'ThisIsAVery8ecureAndLongPa$$word!' | sha3sum -a 256 | cut -d' ' -f1  <= macOS
#  6ab4475aeb3fdde804e60b94ccb3b1996b4bae9751bd0f60cfcdfb0a7f83caab             <= macOS

# export FLASK_IMGAPI__admin__passwordhash="$(echo -n 'ThisIsAVery8ecureAndLongPa$$word!' | sha256sum | cut -d' ' -f1)"

# imgapi.config["IMGAPI"]["token"]
export FLASK_IMGAPI__token="THE_TOO_SIMPLE_PERMANENT_API_TOKEN" # TODO: e.g. a UUID
# export FLASK_IMGAPI__token="$(uuidgen)"


# FYI: JSON Web Token (JWT) is an open standard (RFC 7519) 
#      read https://jwt.io/introduction/ 
# >>> import jwt
# >>> import time
# >>> secretKey = 'FlaskWebServiceApp903!'

# After login, we migth protect the following request-response roundtrips:
# SERVER - FIRST REQUEST 
#      client authenticates with username and password (no token available, up to now)
# SERVER - FIRST RESPONSE create forst token for the client to be used in next request
# >>> now = time.time()
# >>> encoded_jwt = jwt.encode({'user': 'miriam', 'ts': now},secretKey, algorithm='HS256')
# >>> encoded_jwt
# b'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ1c2VyIjoibWlyaWFtIiwidHMiOjE3MDExMDYzNjYuMTI3MTM5OH0.01wdQYqVIf-2eXbOy5BASa8091YJAGsofgLU6EBZTlw'
# create and send (one-time) token to client
# ...
# SERVER  - SECOND REQUEST (the first request was login, where the very first token was created) from the client
# get token from client
# >>> jwt.decode(encoded_jwt,secretKey, algorithms=['HS256'])
# {'user': 'miriam', 'ts': 1701106366.1271398}
# check if user and timestamp are ok... 
#      allow access
#      create new token for next roundtrip (see above)
# ...
# SERVER - SECOND RESPONSE again, create a new one-way token (see above).. and go on (forever)

flask --app imgapi run

