# Using Python to work with Excel

In this example, we use Python to create/manipulate Excel files (_xlsx_).

This is done using the [openpyxl](https://openpyxl.readthedocs.io/en/stable/) library, which has to be installed separately (checkout the [setup.sh](setup.sh)).

> Since Microsoft added native [Python support in Excel](https://support.microsoft.com/en-us/office/get-started-with-python-in-excel-a33fbcbe-065b-41d3-82cf-23d05397f53d), you are also able to work with Python directly in Excel - if your used version of Excel/used license does support it.
>
> At the time this tutorial was created (February 2025), this was not the possible with our available licenses. Maybe in the future we will also add some examples for native Python in Excel directly.

## Prerequisites

Next to Python _(Version > 3.8)_ you need [openpyxl](https://openpyxl.readthedocs.io/en/stable/) which can be installed using `pip`. Therefore we provided a [setup](setup.sh) script, that creates a [virtual environment](https://docs.python.org/3/library/venv.html) and in it the needed dependencies.

> Windows PowerShell users checkout [setup.ps1](setup.ps1)

```console
$ source setup.sh
...
(pyexcel) $ 
```

> _After initial configuration, you can simple load the venv calling `source .venv/bin/activate` (`source .venv/Scripts/activate` on Windows using git-bash; `.\.venv\Scripts\activate` on Windows using PowerShell) to re-activate the virtual environment and therefore have **openpyxl** available._

## Example 1

[data2excel.py](data2excel.py) will create a new Excel document with some data, including some formulas. Feel free to experiment with the used data and checkout the results.

![example data2excel result](.assets/umsatz.png)

## Example 2

[excel-update.py](excel-update.py) will extend the existing [teilnehmer.xlsx](teilnehmer.xlsx) by adding in the next free column the current date and depending on the names provided inside of [teilgenommen.txt](teilgenommen.txt) and entry in form of an **X**. You can run the script multiple times and observe the changes in _teilnehmer.xlsx_.
