#!/usr/bin/env python3 
import sys
import re
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

fn="winemagazine.csv"
if len(sys.argv) >0:
    fn=sys.argv[1]
wine = pd.read_csv(filepath_or_buffer=fn, index_col=0)

def add_year_vintage(dat):
  dat["year"] = dat["title"].apply(lambda x: re.findall(r'\d{4}', x)[:1] )
  dat = dat[ dat["year"].apply(len) > 0  ]
  dat["year"] = dat["year"].apply( lambda x: int(x[0]) )  
  dat = dat[dat["year"] >= 1950] 
  return dat
wine = add_year_vintage(wine)

top_df = wine['variety'].value_counts().head(3)
top_wines = wine[wine['variety'].isin(top_df.index)].copy()

top_italy = top_wines[top_wines["country"] == "Italy"]
italy_ptsvsvariety = top_italy.groupby(["province", "variety"])[["points"]].mean().unstack(fill_value=None).droplevel(0, axis=1)
sns.heatmap(italy_ptsvsvariety, cmap="viridis", annot=True)

plt.savefig("heatmap.png")
