# Secure Coding in Python

Source code with explanations for the **Workshop on Secure Coding** in Python.

## Introduction to Python programming language

* Checkout the [First Semester with Python](https://mode.fh-joanneum.at/secure-code/python-first-semester), unless you are fluent with Python and you can **read code** such as shown in folder [Crashcourse](./0-python-crashcourse/): [Script](./0-python-crashcourse/preview/handout-script.pdf), [OOP](./0-python-crashcourse/preview/handout-oop.pdf)  

* *Note: A printable all-in-one document is available in folder [handout](./handout/): [Secure Coding (PDF in German)](./handout/preview/secure-coding-de.pdf)*

## Starting with (simple) scripts

Creating simple Python scripts. Eexamples to raise security and quality awareness.

* (1a) [Best practices](1a-best_practices/README.md)

* (1b) [Usability](1b-usability/README.md)

* (1c) [Hardcoded information](1c-hardcoded_information/README.md)

* (1d) [Input validation](1d-input_validation/README.md)

* (1e) [Debugging](1e-debugging/README.md) and logging

## Scripts are growing

Managing Source Code with code repositories for backup and collaboration. Running tools such as formatters and linters as well as unit tests to raise code quality.

* (2a) [Managing source code](2a-managing_source_code/README.md) with git

* (2b) [Installing libraries](2b-venv-libs/README.md) into virtual environments (`venv`)

* (2c) [Tools](2c-tools/README.md) for formating, linting, and testing


* (2d) [Project structure](2d-project-structure/README.md) for larger projects


## Special Topics

Selected topics of Python usage in (a,b,c) client / server environments, (d) working with Excel (e.g. exporting data into `csv` or `*.xsl`), and (e,f) using selected libraries for handling large data sets.

### Consuming / providing services

* (3a) [Credential handling](3a-credential_handling/README.md) and client-server communication

* (3b) Consuming [web services](3b-using-web-services/README.md) with examples from GitLab

* (3c) [Simple server](3c-simple_server/README.md) implemented with Flask


* (3d) [Excel](3d-excel/README.md)



### Data Analytics and Data Visualisation

* (3e) [Data visualisation pipeline](3e-visualise-data/README.md) 

* (3f) [Interactive explore](3f-interactive-explore/README.md) large data sets in Jupyter notebooks







> **Note:** *Each of the topics includes not working code or bad examples in the directory (`0_detect`) for raising awareness and for discussion. Code to improve is provided in each subfolder `1_improve`. Finally, one or more possible solutions are given in `2_solution`.*

- - -

Find this [README](https://mode.fh-joanneum.at/secure-code/workshop/-/blob/main/README.md) on <https://mode.fh-joanneum.at/secure-code/workshop>
