#!/usr/bin/env python3
"""
Script to fetch information about existing projects
in GitLab.
Works with empty TOKEN or personal TOKEN
  empty TOKEN => only public projects will be returned
  valid TOKEN => also personal projects will be returned

allows you to search for specific projects when adding search
string on execute, e.g.:
  python3 get_projects.py "What I'm looking for"
"""

import urllib.request
import json
import sys
import os


TOKEN = os.environ.get('API_TOKEN',"") # optional

search = ''

if len(sys.argv) == 2:
    search = sys.argv[1]

# https://docs.gitlab.com/ee/api/projects.html#list-all-projects
api_endpoint = f'https://mode.fh-joanneum.at/api/v4/projects/?search={search}'

# https://docs.gitlab.com/ee/api/rest/#authentication
headers = { 'Authorization': f'Bearer {TOKEN}' }

request = urllib.request.Request(api_endpoint, headers=headers)

try:
    with urllib.request.urlopen(request) as response:
        response_data = response.read().decode('utf8')
        json_data = json.loads(response_data)
        print(json.dumps([{'id': j['id'], 'name': j['name'], 'description': j['description']} for j in json_data], indent=2))
except urllib.error.HTTPError as e:
    print(f"HTTP Error {e.code}: {e.reason}", file=sys.stderr)
except urllib.error.URLError as e:
    print(f"URL Error: {e.reason}", file=sys.stderr)
except Exception as e:
    print(f"unexpected Error: {e}", file=sys.stderr)
