#import "../style/template.typ": *
#import "1a-best-practices.typ": *
#import "1b-usability.typ": *
#import "1c-hardcoded-information.typ": *
#import "1d-input-validation.typ": *
#import "1e-debug.typ": *


// Day 1: Coding / Python Basics

#let intro( selected_lang: "en" ) = {
  if selected_lang == "en" [

  = Prototypical troubles with simple scripts (Awareness)

  ]else [

  = Skripts // Bessere Skripts erstellen


  /*
  Durch Beachtung einiger weniger einfacher Tricks fällt es leicht
    gut bedienbare, gut lesbare und vor allem *gut wartbare Programme* zu schreiben.
  */

  Fast alle Entwickler:innen stehen anfangs vor einigen der folgenden Probleme und stellen sich die Fragen:

- *"How-to?" oder "Wie soll ich beginnen?"*
  Wie kommt man von einem Problem zur Lösungsidee bis hin zum funktionierenden Code? Eine Möglichkeit ist, zuerst Demo-Daten vorzubereiten, dann den Ablauf in einem Skript als Kommentar zu beschreiben. Dann Schritt für Schritt die Kommentare durch echten Code zu ersetzen. Dabei hilft es möglichst nützliche Ausgaben über die aktuellen inneren Zustände während des Programmablaufes zu produzieren oder mittels #english_term([Debugging Breakpoints]) den Code anzuhalten und den Zustand von Variablen zu inspizieren.

- *Wer benötigt Wartbarkeit bei nur wenigen Zeilen Code?*
  Im Sinne der Wartbarkeit ist es von Anfang an besser den Code und auch die Kommentare immer auf englisch zu verfassen. Der Source-Code wird für Menschen geschrieben, somit ist verständliche, sinnvolle Namensgebung extrem wichtig. Schöner Code macht Freude und ist lesbarer. Strukturierter Aufbau und selbsterklärender Code ersparen umfangreiche und mühsame Dokumentation.


- *Was bedeutet Usability bei einem Kommandozeilenprogramm?* #english_term([User Experience]) und #english_term([Usability]) zählen auf mehreren Ebenen! Die Nutzer:innen des Codes sind einerseits die Anwender:innen, aber andererseits auch die Entwickler:innen. Für jene die den Code weiterentwickeln zählt die Lesbarkeit von Programmen, für Endnutzer:innen zählen einfaches Starten, sprechender Output und hilfreiche Fehlermeldungen, sowie auch gut gewählte Default-Werte und einfache Konfiguration.

- *Wieviel Flexibilität ist nötig?* Nutzer:innen wünschen sich unterschiedliche teilweise widersprüchliche Funktionalitäten. Für größtmögliche Flexibilität ist es sinnvoll vorhandene #english_term([default values]) durch Angabe von #english_term([Command-Line Parameter]) überschreiben zu können. Diese Parameter werden beim Start des Skripts angegeben. Daher hat es sich bewährt in einer eigenen Startup Datei -- oft ein einzeiliges Bash Skript -- die Verwendung exemplarisch zu demonstrieren und damit auch zu dokumentieren.

- *Wer sollte den Code wiederverwenden?* Weiterverwendung -- #english_term([copy paste]) -- von existierenden Programmteilen (#english_term([Code Snippets])) ist durchaus wünschens- und empfehlenswert; allerdings nur von Code, den man selbst gut versteht und somit anpassen bzw. abändern kann.


#pagebreak()

Ein paar Gedanken zu *Datensicherheit*, *Einbruchssicherheit*, *Nachvollziehbarkeit* und *Privatsphäre*:

- *Security (Datensicherheit)*: Ist Backup was für Feiglinge? Ein Backup muss nicht mühsam sein. Auch kleine Skripts sollten an einer zentraler Stelle verwaltet und abgelegt werden. So ist jede Änderung nachvollziehbar und ein Programm kann leichter verteilt und aktualisiert werden, bzw. im Falle eines lokalen Datenverlusts wiederhergestellt werden. Use #english_term([Use Git code repositories])!

- *Security (Vertraulichkeit)*: Hard-coding von Passwörtern, Tokens und API Keys lässt sich durch Command-line Parameter und Konfigurationsdateien leicht vermeiden. #english_term([Never hard code credentials])!

- *Security (Hacking)*: Als unbedingte Notwendigkeit wird heute weiters das Prüfen von Benutzereingaben -- Validierung -- angesehen. Durch Vermeidung von gefährlichen API Aufrufen muss die unbeabsichtigte Ausführung von Schadcode verhindert werden. #english_term([Never trust user data])! #english_term([Use Whitelisting for input validation])!

- *Privacy*: Wer kann und darf Zugriff auf *sensible Daten* bekommen? Besser gar keine sinsiblen Daten erheben. Wer liest die Logfiles? Oftmals kann man auch sehr gut mit anonymisierten Daten arbeiten! #english_term([Follow the principle of data minimization])! #english_term([Use anonymisation])!

]


pagebreak()
bestpractices(selected_lang:selected_lang)

pagebreak()
usability(selected_lang:selected_lang)

pagebreak()
hardcoded(selected_lang:selected_lang)


pagebreak()
inputvalidation(selected_lang:selected_lang)

pagebreak()
debug(selected_lang:selected_lang)
}
