#import "../style/template.typ": *

#let usability( selected_lang: "en" ) = {
  if selected_lang == "en" [

  == Usability

  - Solution A Using

  - Solution B Using

  - Solution C Using

  - Solution D Using

  ]else[

  == Benutzbarkeit // Usability

  #section_summary([
    Benutzbarkeit (Usability) ist auch im Terminal für Konsolenapplikationen wichtig.
    User erwarten *hilfreiches Feedback* über die Art und Umfang der Erwarteten Eingaben,
    über den Systemzustand,
    über den aktuellen Fortschritt oder eines auftretenden Fehlers.
  ])
  /*Dadurch werden Benutzer:innen ermächtigt sich jederzeit selbst zu helfen.*/

  #markup([Problem], [
    Programme sind leider oft schwer oder umständlich zu bedienen.
    Vor allem, wenn die Verwendung nicht einfach, schnell und intuitiv ist.
    Zum Beispiel kann die Benutzbarkeit (Usability) durch fehlende Grundeinstellungen
    oder unverständliche Rückmeldungen für die Nutzer:innen niedrig sein. Dadurch werden
    diese Programme dann nicht gerne benutzt.
  ])

  /*    Positive _User Experience_ bereitet Freude, so eine Applikation wird gerne benutzt. */

  #pysrc_with_overlay("1b/no.py", [Please Improve])

  - *Empfehlung A: Sprechende Namensgebung*
    Der Name eines Skripts (einer Applikation) sollte selbsterklärend sein.

    #bash("1b/down-snippet.sh") // ./image_downloader.py urls.txt

  - *Empfehlung B: Bereitstellen eines Start Skripts*. Ein Shell Start Skript soll vorhanden sein, damit die Verwendung klar ersichtlich ist. Vor allem, wenn Kommandozeilen Parameter nötig sind haben die User ein funktionierendes Beispiel zur Hand. Weiters ist das Bereitstellen von geeigneten Default-Werten hilfreich, weil es ebenfalls die Verwendung vereinfacht.

    #bashscript("1b/startscript-snippet.sh")

  - *Empfehlung C: README*. Ein — durchaus kurzes — Markdown Textfile mit Informationen über den Zweck und die Verwendung eines Programmes (`README.md`) wird auf Git direkt dargestellt. Bereitgestellte Links können angeklickt werden.

    #table(
      columns: (60%,40%),
      inset: 0pt, column-gutter: 0.3cm, stroke: none,
      align: horizon,

      textsrc("1a/readme-snippet.md", title: "README.md", lang: "Markdown"),
      // sourcecode(lang: "Markdown", raw( read("../code/1a/readme-snippet.md") )),
      box( stroke: 1pt + gray,
        image("../images/readme.png", fit: "contain", width: 87%))
    )

  #pagebreak()

  - *Empfehlung D: Feedback* über Zustand und über Fehler durch Ausgabe des aktuellen Status. Der aktuelle Zustand kann durch einfache Print Ausgaben mitgeteilt werden. Fehlerzustände durch konstruktive, lösungsorientierte Fehlermeldungen.

    #pysrc_with_ok("1b/feedback-snippet.py")

  - *Empfehlung E: Abbruch bei Fehlern*. Wenn nach dem Auftreten eines Fehlers nicht sinnvoll weitergearbeitet werden kann, ist es ok, bei solchen Fehlern einfach ein Programm abzubrechen. #english_term([Let it crash approach])! *Erwartbare Fehler abfangen*: jene -- erwartbaren -- Fehler, die durch fehlerhafter Daten (z.B.: ungültige URL) oder falsche Eingaben (z.B.: Tippfehler) entstehen, müssen aber sehr wohl abgefangen werden. Nach der Fehlerbehandlung (z.B.: Fehler in ein Logfile schreiben) kann das Programm sinnvoll weiterlaufen.

    #pysrc_with_ok("1b/errorhandling-snippet.py")


  #markup([Problem], [
    Die Benutzbarkeit, die _Usability_ sollte für Entwickler:innen immer möglichst hoch sein.
  ])

  - *Empfehlung A: Sprechende Namensgebung*. Bei Variablen, Funktionen, Klassen und Methoden sollen sprechende, selbsterklärende Namen auf englisch verwendet werden. Dabei ist die Python übliche Schreibweise laut #english_term([styleguides])#footnote(["StyleGuides" #link("https://pep8.org")]) zu beachten.

    #pysrc_with_ok("1b/naming-snippet.py")


  - *Empfehlung B: Effizienter durch Python-artigen Code*. Das Verwenden von Python-typischem Code hilft kurze, prägnante und wartbare Programme zu schreiben.  #english_term([Write idiomatic Python code])!

    Beispielsweise könnte man umfangreiche Daten in Listen `list` und Dictionaries `dict` verwalten, sortieren und filtern, oder mit einem Set `set` Duplikate entfernen.  #english_term([Thinking in lists])!

    #pysrc_with_ok("1b/pythonic-snippet.py")

  ]
}
