#import "../style/template.typ": *

#let hardcoded( selected_lang: "en" ) = {
  if selected_lang == "en" [

== Hard Coded Information <neverhardcode>

#section_summary([])

#markup([Problem], [
  Sensitive data is hard coded into the source code.
  Even worse, credentials might be output and sensitive information appear in logs.
])

#pysrc_with_overlay("1c/no.py", [Please Improve])


- Solution A Using command line arguments

  #sourcecode(lang: "Python", raw( read("../code/1c/cli-args-snippet.py") ))


- Solution B Using environment variables

  #sourcecode(lang: "Python", raw( read("../code/1c/env-snippet.py") ))


- Solution C Using configuration files (here _config.py_ containing ip="10.0.0.7"):

  #sourcecode(lang: "Python", raw( read("../code/1c/config-usage-snippet.py") ))

- Solution D Using Default values

  #sourcecode(lang: "Python", raw( read("../code/1c/defaults-snippet.py") ))

  *Further Suggestions:*

  (a) If a users does not specify required command line arguments, *provide helpful error messages* to explain how to pass along the necessary arguments, for example output `Usage ./downloader.py -p <pwd>`.

  (b) When using configuration files, provide a *template* file with demo data. This file has to be copied and edited to insert the concrete values which should be read in by the programme.

  (c) Note to add the template files to the Git repository. But *exclude* the concrete files *from the repo* by adding the names of the local configuration files to `.gitignore`.


  ]else[

  == Passwörter // "credentials" niemals im Source

  #section_summary([
    Idealerweise erhalten Python Skripts ihre Daten von extern, sind also nicht fest in den Source Code hineinkodiert. Skripts sind ebenfalls über externe Settings konfigurierbar. Sensiblen Daten wie Passwörter landen auf keinen Fall im Code. *#english_term([Never hard code credentials])!*
  ])


  #markup([Problem], [
    Sensible Daten (wie Passwörter oder auch API keys) finden sich als Teil des Source Codes.
    Schlimmstenfalls werden sensible Informationen sogar ausgegeben und geheime Passwörter
    tauchen später in Log Dateien auf.
  ])

  #pysrc_with_overlay("1c/no.py", [Please Improve])

  - *Lösung A: Kommandozeilenargumente*. Übergabe von Konfigurationen als _command-line parameters_.

    #pysrc_with_ok("/1c/cli-args-snippet.py")

  - *Lösung B: Umgebungsvariable*. Übergabe von Konfigurationen mittels _environment variables_ (Dies ist vor allem im Umfeld von CI/CD -- siehe @git Git -- üblich).

    #pysrc_with_ok("1c/env-snippet.py")

  - *Lösung C: Konfigurationsfile*. Festlegen von Konfigurationen in einem _config-file_. Hier zum Beispiel _config.py_ mit dem Inhalt `ip="10.0.0.7"`.

    #pysrc_with_ok("1c/config-usage-snippet.py")

  - *Lösung D: Standardwerte*. Das Festlegen von gut gewählten _default values_ erübrigt in vielen Fällen die explizite Übergabe von Startwerten.

    #pysrc_with_ok("1c/defaults-snippet.py")

  #pagebreak()

  - *Lösung E: Pipes*. Werte können auch über eine sogenannte _Pipe_ an ein Programm übergeben werden.


    #pysrc_with_ok("1c/./data_processing-snippet.py",
      title: "data_processing.py")

    bzw.

    #bashscript("1c/./data_processing-startup.sh", title: "startup.sh")


  *Weitere Empfehlungen:*

  - Bei fehlenden Kommandozeilen Parametern sollte man User zur Angabe gültiger Werte durch *hilfreiche Fehlermeldungen* hinleiten, z.B. `Usage ./downloader.py <urls> <target_dir>`.

  - Bei Konfigurationsfiles ist es günstig eine *Templatedatei*, z. B.: `config_template.yaml`, als Muster mit Demo-Daten bereitzustellen, welches vor bei der Verwendung umbenannt und mit konkreten Werten versehen wird.

  - Im Git Repository sollten nur Vorlagen gespeichert werden, die jeweiligen lokalen Konfigurationsdateien werden mittels `.gitignore` vom *Repo ausgenommen*.
  ]


}
