// Secure Coding Handout template
#import "@preview/codelst:2.0.2": sourcecode

// 2023, IIT (FHJ)
#let todo(title,term,color:red) = {
  text(color,[#title: ])
  text(blue, [#term])
}
#let markup(title,body) = {
  text(black, [
    #v(0.5cm)
    *#title:*
  ])
  body
  v(0.2cm)
}
#let section_summary(body) = {
  box(
    inset: 1.3em,
    fill: luma(230),
    text(style:"italic", par(hanging-indent: 4.2em, [Summary: ]+body)))
}

// highlight enlish terms, such as: overengineering
#let english_term(body) = {
  text(style:"italic",["#body"])
}
#let pykeyword(body) = {
  text(font: "New Computer Modern", style:"normal", body)
}

#let handout(
  date: none,
  selected_lang: "en",
  body
) = {
  set text(font: "PT Sans") //  style: "normal", weight: "regular")
  set text(lang: selected_lang)
  // Configure page
  set page( "a4",
    margin: (top: 3cm),
    header: context {
      // current page number
      let pageNumber = counter(page).at(here()).first()

      // array /list of all the top-level headings
      let headings = query(heading.where(level: 1))
      let subheadings = query(heading.where(level: 2))

      // is this page the first page within a section?
      let isSectionPage = headings.any(it => it.location().page() == pageNumber)
      let isSubSectionPage = subheadings.any(it => it.location().page() == pageNumber)

      let before_headings = query(selector(heading.where(level: 1)).before(here()))
      if before_headings.len() == 0 {
        // We ignore any 0 entry (i.e. on title page, on TOC page
        // curr_heading = query(selector(heading.where(level: 1)).after(here())).first()
        // [FIRST]
        // curr_heading.body
      } else {
          let curr_heading = before_headings.last()
          if isSectionPage {
            // we display no Header for a NEW section (i.e. a Section Page)
          }else{
            // Top left we display in the header the Section number UNLESS subsection starts here
            // Top left we display in the header teh Section name
            if  not isSubSectionPage {
              counter(heading).display("1.a.1") // is 3b even if we currently show 3a
              h(0.4em)
            }
            curr_heading.body // Selected Topics "
          }
      }
    },
    footer: context {
      let pageNumber = counter(page).at(here()).first()
      if pageNumber>1 {
        [ www.fh-joanneum.at/iit
          #h(1fr)
          Seite #counter(page).display("1")
        ]
      }
    },
    number-align: center,
  )

  // Defining some styles:
  //  Put every new chapter on a new page:
  show heading.where(level:1): title =>{
    [#title]
    v(.8em) // some space after each section heading
  }

  // Title page

  set align(center)
  v(3cm)
  image("../images/logo.svg", width: 50%)
  v(7cm)
  text(28pt,
  if selected_lang == "en" [
    Secure Coding in Python
  ]else[
    Sicheres Programmieren in Python
  ]
  )


  if selected_lang == "en" [

    A guide for better programming

  ]else[

    Tipps und Tricks

  ]
  v(1fr)
  // latest version information: "Printed February 7, 2027"
  align(center, text(size:10pt, weight: "light", date))

  set align(left)
  set par(justify: true)

  // Table of contents Level 1: "Days" 1, 2, 3,
  show outline.entry.where(
    level: 1
  ): it => {
    v(14pt, weak: true)
    strong(it)
  }
  // Table of contents Level 2 "Topics" 2a, 2b
  show outline.entry.where(
    level: 2
  ): it => {
    v(12pt, weak: true)
    h(0.5cm)
    it
  }

  pagebreak()

  outline(indent: 0.32cm, depth: 2 )
  // Remainder of the document / Body
  body
}

// Different templates for Code Listings and Filecontents:

#let textsrc(
  filename,
  title:none,
  lang: "Text"
) = {
    let path_to_src="../code/"+filename
    box([
        #sourcecode(
          lang: "Text",
          raw( read(path_to_src) ),
          numbers-style: (lno) => "",
        )
        #place(
          top+right,dy:-0.5em,
          block(
              fill: rgb(100, 100, 100, 100), outset: 0.1cm, radius: 0.1cm,
              [#h(0.5cm) #title #h(0.5cm)]
            )
        )
    ])
}
#let bash(
  filename) = {
    let path_to_src="../code/"+filename
     box([
        #sourcecode(
          lang: "Bash",
          raw( read(path_to_src) ),
          numbers-style: (lno) => "> ",
        )
        #place(dx:-0.3em,dy:-0.5em,
          bottom+right,
            text(size: 14pt, fill: red, [✔︎])
        )
     ])
}
#let bashscript(
  filename, title:none,) = {
    let path_to_src="../code/"+filename
    box([
      #sourcecode(
        lang: "bash",
        raw( read(path_to_src) ),
      )
      #place(dx:-0.3em,dy:-0.5em,
        bottom+right,
          text(size: 14pt, fill: red, [✔︎])
      )
      #if title != none {
        place(
          right, dy:-1.2cm,
          block(
              fill: rgb(100, 100, 100, 100), outset: 0.1cm, radius: 0.1cm,
              [#h(0.5cm) #title #h(0.5cm)]
            )
        )
      }
    ])
}
#let pysrc_with_ok(
  filename,
  title:none,
  start_with_number: 1,
) = {
  let path_to_src="../code/"+filename
  box( [
      #sourcecode(lang: "Python", raw( read(path_to_src) ), numbers-start: start_with_number)
      #if title != none {
        place(
          top+right, dy:-0.5em,
          block(
              fill: rgb(100, 100, 100, 100), outset: 0.1cm, radius: 0.1cm,
              [#h(0.5cm) #title #h(0.5cm)]
            )
        )
      }
      #place(dx:-0.3em,dy:-0.5em, bottom+right, text(size: 14pt, fill: red, [✔︎]))
    ])
}

#let pysrc_with_overlay(
  filename,
  overlay_text,
  ) = {
    let path_to_src="../code/"+filename
    box([
        #sourcecode(lang: "Python", raw( read(path_to_src) ))
        #place(right+bottom,dx:-0.3em,dy:-0.5em,text(size: 8pt, fill: red, [❌]))
        #place(
          right+top,
          dx: 0cm,
          rotate(20deg,
            block(
              fill: red, outset: 0.4cm, radius: 0.3cm,
              overlay_text
            )
          )
        )
    ])
}
